#ifndef _ABON_INTERFACE_H
#define _ABON_INTERFACE_H

#include "classifier.h"
#include "timer.h"

#ifdef __cplusplus
extern "C" {
#endif

enum abonErrorCode
{
	aboneOk = 0,
	aboneParam, //spatny parametr
	aboneSize, //obrzek je jinak velk, ne bylo nastaven abonSetSize
	aboneError, //obecna chyba
	aboneNumErrorCodes
};

enum abonTimeStamps
{
	abontsTotal,
	abontsPyramid,
	abontsConvolution,
	abontsDetection,
	abontsAux1,
	abontsAux2,
	abontsNumTimeStamps // totok musi byt posledni
};

//Forward deklarace struktur

struct abonDetector;
struct abonImageData;
struct abonTimeStamp;
struct abonDetection;

//Callbacky

typedef abonImageData *abonCbGetImage(void *param);
typedef abonErrorCode abonCbPutImage(void *param, abonImageData *image);

//Rozhrann knihovny

//Knihovna exportuje jen abonCreate
typedef abonDetector *abonFunCreate();

//Ukazatele na tyhle funkce nacpe abonInit do struktury abonDetector
typedef abonErrorCode abonFunSetVerbosity(abonDetector *detector, unsigned verbosity);
typedef abonErrorCode abonFunSetTimer(abonDetector *detector, Timer *timer);
typedef abonErrorCode abonFunSetClassifier(abonDetector *detector, TClassifier *classifier);
typedef abonErrorCode abonFunSetStep(abonDetector *detector, unsigned hstep, unsigned vstep);
typedef abonErrorCode abonFunSetSize(abonDetector *detector, unsigned width, unsigned height, unsigned pitch);
typedef abonErrorCode abonFunProcess(abonDetector *detector, abonImageData *image);
typedef abonErrorCode abonFunStart(abonDetector *detector, abonCbGetImage *get, abonCbPutImage *put, void *param);
typedef abonErrorCode abonFunRelease(abonDetector *detector);
typedef abonErrorCode abonFunTimestampDescription(abonDetector *detector, abonTimeStamps *timestamp, const char **description);

struct abonDetector
{
	void *library;//< dll/so handle, set by abonLoad
	abonFunCreate *create;
	//nastaven detektoru
	abonFunSetVerbosity *setVerbosity;
	abonFunSetTimer *setTimer;
	abonFunSetClassifier *setClassifier;
	abonFunSetStep *setStep;
	abonFunSetSize *setSize;
	//zpracovn
	abonFunProcess *process;
	abonFunStart *start;
	//klid
	abonFunRelease *release;
	abonFunTimestampDescription *timestampDescription;
	//nastaven hodnoty
	unsigned verbosity;
	Timer *timer;
	TClassifier *classifier;
	unsigned hstep, vstep;
	unsigned width, height, pitch;
	//vlastn vci si knihovna pid za konec
};

/// abonDetector initial value. Use instead of memset in plugins.
const abonDetector abonDetectorInitializer = {
	NULL, NULL,//handle, create
	NULL, NULL,//verbosity, timer
	NULL, NULL, NULL,//klasifiktor, velikost, krok
	NULL, NULL,//process, start
	NULL, NULL,//klid, popis
	//hodnoty
	0, NULL,//verbosita, timer
	NULL,//klasifiktor
	1, 1,//krok
	0, 0, 0,//velikost
};

struct abonTimeStamp
{
	double start;
	double end;
};

struct abonDetection
{
	unsigned x,y;//levy horni roh
	unsigned width,height;
	float response;
	float angle;

#ifdef __cplusplus
	abonDetection() {}
	abonDetection(unsigned _x, unsigned _y, unsigned _w, unsigned _h, float _r, float _a)
	: x(_x), y(_y), width(_w), height(_h), response(_r), angle(_a) {}
#endif
};

struct abonImageData {
  // image data
  unsigned width, height, pitch;
  char *data;

  // other stuff
  unsigned imageID;

  // timestamps
  abonTimeStamp timestamps[abontsNumTimeStamps];

  // vysledne detekce
  unsigned detectionCount;
  abonDetection *detections;
  bool nmsDone;//true - non-maxima suppression done
  
  // dalsi veci???
};

#ifdef __cplusplus
}
#endif

#endif
