/*
 *   classifier.h
 *   $Id: classifier.h 30 2009-03-24 10:17:12Z ijosth $
 */

#ifndef _ABON_CLASSIFIER_H_
#define _ABON_CLASSIFIER_H_

#include <iostream>

/// A single weak hypothesis.
/// 3x3 LRD feature, WaldBoost threshold and ptr to lookup table.
/// NOTE: 'alpha' should be indexed by values [-8..8], NOT [0..17]!!!
struct TStage
{
	// Feature and hypothesis parameters
    int x, y, w, h;
    char A, B;      ///< Rank indexes
	float theta_b;  ///< Wald_boost threshold
	float * alpha;  ///< Ptr to table with alphas
    
    // Additional parameters
    char szType;  // 2 * (h-1) + (w-1)
    char posType; // (4 * (y % 4) + (x % 4))
};


/// Classifier structure.
/// Holds informations about a classifier - parameters,
/// weak hypotheses, alphas and convolutions.
struct TClassifier
{
    unsigned stageCount;          ///< Total Number of stages
    unsigned evaluatedStageCount; ///< Max stages to evaluate
    unsigned alphaCount;          ///< Number of alphas per stage
    float threshold;              ///< Final classification threshold (typically 0.0f)
    unsigned width, height;       ///< Size of scanning window
    TStage * stage;               ///< List of stages
    float * alpha;                ///< List of alphas
};


/// Initialize classifier structure.
/// Links stages to alphas.
/// \param c Classifier to init
int initClassifier(TClassifier * c);

/// Load classifier from XML.
/// \param root Root element of the classifier
/// \returns Ptr to initialized classifier (no need to call initClassifier())
TClassifier * loadClassifierXML(const char * filename);

void releaseClassifier(TClassifier ** c);

/// Export classifier to .h file
void exportClassifierHeader(TClassifier * c, std::ostream & str, const char * name);

class TClassifierReleaser
{
private :
	TClassifier *classifier;
public :
	TClassifierReleaser(TClassifier *c) : classifier(c) {}
	~TClassifierReleaser() { releaseClassifier(&classifier); }
};

#endif

