# ABon common part of makefile for VC++/IC++
# $Id: Makefile.inc 73 2009-04-09 11:29:59Z ihavel $

LINKLIB=$(LINKLIB) $(**) /OUT:$@ $(LIBFLAGS)
LINKEXE=$(LINK) $(**) /OUT:$@ $(LINKFLAGS)
LINKPLUGIN=$(LINK) $(**) /OUT:$@ $(LINKFLAGS) /DLL /IMPLIB:$(LIBD)\$(@B).lib /DEF:$(SRCD)\abon.def
MANIFESTEXE=mt /nologo -manifest $@.manifest -outputresource:$@;1
MANIFESTDLL=mt /nologo -manifest $@.manifest -outputresource:$@;2

# plugin core library
CORE=$(LIBD)\libcore.lib
LAME=$(LIBD)\liblame.lib

# compiled face detector
FACEDET=$(INCD)\facedetector.h

# programs
XML2H=$(BIND)\xml2h.exe
DETECT=$(BIND)\detect.exe

# plugins
DUMMY=$(BIND)\libdummy.dll
SIMPLE=$(BIND)\libsimple.dll
SSE=$(BIND)\libsse.dll
IMADEV=$(BIND)\libimadev.dll
GPU=$(BIND)\libgpu.dll
CUDA=$(BIND)\libcuda.dll
CORNER=$(BIND)\libcorner.dll

all: detect dummy simple sse imadev cuda corner

core: $(CORE)
lame: $(LAME)
xml2h: $(XML2H)
detect: $(DETECT)
dummy: $(DUMMY)
simple: $(SIMPLE)
sse: $(SSE)
imadev: $(IMADEV)
gpu: $(GPU)
cuda: $(CUDA)
corner: $(CORNER)

cleantmp:
	@echo "Cleaning"
	@$(RM) *.linkinfo
	@$(RM) $(SRCD)\*.obj
	@$(RM) $(SRCD)\libdummy\*.obj
	@$(RM) $(SRCD)\libsse\*.obj
	@$(RM) $(SRCD)\libimadev\*.obj
	@$(RM) $(SRCD)\libgpu\*.obj
	@$(RM) $(SRCD)\libgpu\berLame_src\*.obj
	@$(RM) $(SRCD)\libcuda\*.obj
	@$(RM) $(SRCD)\libcorner\*.obj
	@$(RM) $(BIND)\*.manifest
	@$(RM) $(LIBD)\*.lib
	@$(RM) $(LIBD)\*.exp
	@$(RM) $(FACEDET)

clean: cleantmp
	@$(RM) $(BIND)\*.dll
	@$(RM) $(BIND)\*.exe

# plugin core library
$(CORE): $(SRCD)\timer.obj
	@echo "Linking core"
	@$(LINKLIB)

# berlame library
ULS=$(SRCD)\libgpu\berLame_src
$(LAME): $(ULS)\BitArray.obj $(ULS)\CallStack.obj $(ULS)\DCT.obj $(ULS)\Hash.obj $(ULS)\Jpeg.obj $(ULS)\OpenGL20.obj $(ULS)\OpenGLDrv.obj $(ULS)\OpenGLState.obj $(ULS)\Shader2.obj $(ULS)\Texture.obj $(ULS)\TextureUtil.obj $(ULS)\Tga.obj $(ULS)\Vector.obj $(ULS)\RenderBuffer2.obj $(ULS)\VertexBufferObject.obj $(ULS)\ShaderSys.obj $(ULS)\StlUtils.obj $(ULS)\Timer.obj $(ULS)\dev\GPUImgProc.obj
	@echo "Linking lame"
	@$(LINKLIB)

# XML to .h converter
$(XML2H): $(SRCD)\xml2h.obj $(SRCD)\classifier.obj $(SRCD)\simplexml.obj
	@echo "Linking xml2h"
	@$(LINKEXE) libxml2.lib
	@$(MANIFESTEXE)

# embedded classifier
$(FACEDET): $(XML2H) $(DATD)\facedetector.xml
	@echo "Converting facedetector"
	@$(**) -o $@ -n facedetector

# detector executable
$(DETECT): $(CORE) $(SRCD)\detect.obj $(SRCD)\abon.obj $(SRCD)\config.obj $(SRCD)\io.obj $(SRCD)\input.obj $(SRCD)\classifier.obj $(SRCD)\simplexml.obj
	@echo "Linking detect"
	@$(LINKEXE) libxml2.lib libargtable2.lib pthreadVC2.lib cxcore.lib cv.lib highgui.lib
	@$(MANIFESTEXE)

# example plugin
$(DUMMY): $(CORE) $(SRCD)\libdummy\libdummy.obj
	@echo "Linking dummy"
	@$(LINKPLUGIN)
	@$(MANIFESTDLL)

# simple evaluator
$(SIMPLE): $(CORE) $(SRCD)\libsimple\libsimple.obj $(SRCD)\libsimple\lrd_engine.obj $(SRCD)\libsimple\image.obj
	@echo "Linking simple"
	@$(LINKPLUGIN)
	@$(MANIFESTDLL)

# parallel sse evaluator
SSES=$(SRCD)\libsse
$(SSE): $(CORE) $(SSES)\libsse.obj $(SSES)\lrd_const.obj $(SSES)\lrd_engine.obj $(SSES)\imagetools.obj $(SSES)\scale.obj
	@echo "Linking sse"
	@$(LINKPLUGIN)
	@$(MANIFESTDLL)

# dsp+fpga evaluator
$(IMADEV): $(CORE) $(SRCD)\libimadev\libimadev.obj $(SRCD)\libimadev\Uni1pUtils.obj
	@echo "Linking imadev"
	@$(LINKPLUGIN) libUni1P.lib
	@$(MANIFESTDLL)

# gpu evaluator
GPUS=$(SRCD)\libgpu
$(GPU): $(CORE) $(LAME) $(GPUS)\DetectGPU.obj $(GPUS)\Convolve.obj $(GPUS)\AsciiHist.obj $(GPUS)\InitOpenGL.obj $(GPUS)\Load_XML.obj $(GPUS)\OpenGL20Logo.obj $(GPUS)\WaldBoost.obj 
	@echo "Linking gpu"
	@$(LINKPLUGIN) gdi32.lib user32.lib
	@$(MANIFESTDLL)

# cuda evaluator
CUDAS=$(SRCD)\libcuda
$(CUDA): $(CORE) $(CUDAS)\lrdlib.obj $(CUDAS)\lrdclass.obj $(CUDAS)\GLee.obj $(CUDAS)\counter.obj $(CUDAS)\lrd.obj
	@echo "Linking cuda"
	@$(LINKPLUGIN) cudart.lib cutil32.lib
	@$(MANIFESTDLL)

# OpenCV corner detector
$(CORNER): $(CORE) $(SRCD)\libcorner\libcorner.obj
	@echo "Linking corner"
	@$(LINKPLUGIN) cxcore.lib cv.lib
	@$(MANIFESTDLL)

# dependencies

$(INCD)\abon.h: $(INCD)\aboninterface.h
$(INCD)\aboninterface.h: $(INCD)\classifier.h $(INCD)\timer.h

$(SRCD)\abon.cpp: $(INCD)\abon.h
$(SRCD)\timer.cpp: $(INCD)\timer.h
$(SRCD)\classifier.cpp: $(INCD)\classifier.h
$(SRCD)\simplexml.cpp: $(INCD)\simplexml.h

$(SRCD)\xml2h.cpp: $(INCD)\classifier.h $(INCD)\simplexml.h

$(SRCD)\io.h: $(SRCD)\input.h
$(SRCD)\input.h: $(INCD)\abon.h $(SRCD)\config.h
$(SRCD)\config.cpp: $(SRCD)\config.h
$(SRCD)\input.cpp: $(SRCD)\input.h
$(SRCD)\detect.cpp: $(FACEDET) $(SRCD)\io.h

$(SRCD)\libdummy\libdummy.cpp: $(INCD)\aboninterface.h

$(SRCD)\libsimple\lrd_engine.h: $(INCD)\aboninterface.h
$(SRCD)\libsimple\image.cpp: $(SRCD)\libsimple\image.h
$(SRCD)\libsimple\lrd_engine.cpp: $(SRCD)\libsimple\lrd_engine.h
$(SRCD)\libsimple\libsimple.cpp: $(SRCD)\libsimple\image.h $(SRCD)\libsimple\lrd_engine.h

$(SRCD)\libsse\lrd_engine.h: $(INCD)\aboninterface.h $(SRCD)\libsse\imagetools.h
$(SRCD)\libsse\libsse.h: $(SRCD)\libsse\lrd_engine.h
$(SRCD)\libsse\imagetools.cpp: $(SRCD)\libsse\imagetools.h
$(SRCD)\libsse\lrd_const.cpp: $(SRCD)\libsse\lrd_const.h
$(SRCD)\libsse\lrd_engine.cpp: $(SRCD)\libsse\lrd_const.h $(SRCD)\libsse\lrd_engine.h
$(SRCD)\libsse\scale.cpp: $(SRCD)\libsse\scale.h
$(SRCD)\libsse\libsse.cpp: $(SRCD)\libsse\libsse.h $(SRCD)\libsse\scale.h $(SRCD)\libsse\cpuid.h

$(SRCD)\libimadev\Uni1pUtils.cpp: $(SRCD)\libimadev\Uni1pUtils.h
$(SRCD)\libimadev\libimadev.cpp: $(INCD)\aboninterface.h $(SRCD)\libimadev\Uni1pUtils.h $(SRCD)\libimadev\comm.h

$(SRCD)\libcuda\lrd.cu: $(SRCD)\libcuda\lrdkernel.cu

$(SRCD)\libcorner\libcorner.cpp: $(INCD)\aboninterface.h

# rules

.c.obj:
	@$(CC) $(CFLAGS) /c /Fo$@ $(**)

.cpp.obj:
	@$(CPP) $(CPPFLAGS) /c /Fo$@ $(**)

.cu.obj:
	@"$(CUDA_BIN_PATH)\nvcc.exe" $(NVCCFLAGS) -arch sm_11 -ccbin "$(VCINSTALLDIR)\bin" -c -I"$(CUDA_INC_PATH)" -I./ -I"$(NVSDKCUDA_ROOT)/common/inc" -o $@ $(**)

.SUFFIXES: cu
