//-----------------------------------------------------------------------------
// detect.cpp
// Command line parsing
// Jiri Havel, DCGM FIT BUT, Brno
// $Id: detect.cpp 58 2009-04-02 09:06:59Z ijuranek $
//-----------------------------------------------------------------------------

#include "io.h"

// STL
#include <fstream>
#include <iostream>
#include <memory>
#include <vector>

// Command line parser
//#include <argtable2.h>

// Built in classifier
#include <facedetector.h>

using namespace std;

/*bool parseCommandFile(Config & config, const char * filename);
bool parseCommandLine(Config & config, int argc, char ** argv);*/

int main(int argc, char *argv[])
{
	// Set default configuration
	Config config;
	config.engine = "gpu";
	config.threshold = 0.0;
	config.stages = 0;
	config.noearly = false;
	config.hstep = 1;
	config.vstep = 1;
	config.groupThreshold = 0.4;
	config.groupMinCount = 2;
	config.smooth = 0;
	config.scaleBase = 1.0;
	config.fourcc = CV_FOURCC('X', 'V', 'I', 'D');
	config.tag = config.show = config.drawAll = false;
	config.verbose = true;
	config.start = config.frames = 0;
	config.step = 1;
	config.show = true;

#if 0
	for(int i = 0; i < 10; ++ i) {
		Config::Input inp(Config::Input::IMAGE,
			"E:\\MyProjects\\z_large_datafiles\\GPU_WaldBoost_mark2\\test3.jpg");
		config.inputs.push_back(inp);
	}
#else
	Config::Input inp(Config::Input::VIDEO,
		"E:\\MyProjects\\z_large_datafiles\\GPU_WaldBoost_mark2\\Basement Jaxx - Where's Your Head At.mpg");
#endif

	// Parse command line arguments
/*	if(!parseCommandLine(config, argc, argv))
	{
        return EXIT_FAILURE;
	}*/

	// Optionally show config
	if(config.verbose)
	{
		cout << "Configuration:\n" << config;
	}

	// Create detector
	abonDetector *detector = abonLoad(config.engine.c_str());
	if(detector == NULL)
	{
        return EXIT_FAILURE;
	}
	DetectorPtr detector_unloader(detector);

	// Load classifier
	TClassifier *classifier = &facedetector;
	/*if(!config.classifier.empty())
	{
		if((classifier = loadClassifierXML(config.classifier.c_str())) == NULL)
		{
			cerr << "Can't load classifier file '" << config.classifier << "'!\n";
			return EXIT_FAILURE;
		}
	}
	TClassifierReleaser(config.classifier.empty() ? NULL : classifier);*/

    // Initialize classifier
	if(initClassifier(classifier) == 0)
	{
		cerr << "Can't init classifier!" << endl;
		return EXIT_FAILURE;
	}
    classifier->threshold = static_cast<float>(config.threshold);//final threshold
	// Set evaluated stage count
	if(config.stages > 0)
	{
		if(config.stages > classifier->stageCount)
		{
			cerr << "Requested stage count is greater than classifier length!\n";
			return EXIT_FAILURE;
		}
		classifier->evaluatedStageCount = config.stages;
	}
	// Disable waldboost(early termination)
    if(config.noearly)
	{
		for(unsigned i = 0; i < classifier->stageCount; ++i)
			((TStage*)(classifier->stage) + i)->theta_b = -1e+010;
	}

	// Connect to database
/*	if(!config.database.empty())
	{
		if(config.verbose) cout << "Connecting... (" << config.database << ")\n";
		if(
			((config.db = vplDbInit(NULL, config.database.c_str(), config.datapath.c_str())) == NULL) ||
			(config.db->conn == NULL))
		{
			cerr << "Can't connect to database!\n";
			goto error;
		}
		//list datasets
		if(config.verbose)
		{
			vplDbDatasetsPrint(config.db);
		}
		//open dataset
		if((config.seqs = vplDbSequencesList(config.db, config.dataset.c_str())) == NULL)
		{
			cerr << "Can't open dataset '" << config.dataset << "'!\n";
			goto error;
		}
		//list sequences
		if(config.verbose)
		{
			vplDbSequencesPrint(config.db, config.dataset.c_str());
		}
	}
*/
	// Output window
	if(config.show) 
	{
		cvNamedWindow("Output");
	}

	// Init timer
	TimerPtr timer;
	if(timer.timer == NULL)
	{
		cerr << "Can't create timer!\n";
		return EXIT_FAILURE;
	}

	// Set detector parameters
	if(abonSetVerbosity(detector, config.verbose ? 1 : 0) != aboneOk)
	{
		cerr << "Can't set verbosity!\n";
		return EXIT_FAILURE;
	}
	if(abonSetTimer(detector, timer.timer) != aboneOk)
	{
		cerr << "Can't set timer!\n";
		return EXIT_FAILURE;
	}
	if(abonSetClassifier(detector, classifier) != aboneOk)
	{
		cerr << "Can't set classifier!\n";
		return EXIT_FAILURE;
	}
	if(abonSetStep(detector, config.hstep, config.vstep) != aboneOk)
	{
		cerr << "Can't set step!\n";
		return EXIT_FAILURE;
	}

	InputOutput io(config, detector, timer.timer);
	
	// Process inputs
	for(unsigned i = 0; i < config.inputs.size(); ++i)
	{
		if(config.verbose) cout << "Processing " << config.inputs[i] << " :\n";
		std::auto_ptr<Input> input(inputCreate(config, i));
		if(input.get() == NULL)
		{
			cerr << "Can't open '" << config.inputs[i] << "'!\n";
			return EXIT_FAILURE;
		}
		std::auto_ptr<Output> output(outputCreate(config, i));
		io.runWithIO(input.get(), output.get());

		//getchar(); // debug
	}

	if(config.verbose) cout << "Total time " << timer.time() << "s.\n";

	return EXIT_SUCCESS;
}

/*
void drawDetections(IplImage * inputFrame, TDetectionList::iterator first, TDetectionList::iterator last,
        const CvScalar & color, bool outline, bool printResponse, const CvFont * font)
{
    for (TDetectionList::iterator i = first; i != last; ++i)
    {
        TDetection & d = *i;
        CvRect r = d.area;
        float angle = d.angle;
        float response = d.response;

        r.x += r.width/2;
        r.y += r.height/2;

        // Prepare rotation matrix
        float _negRot1M[6];
        CvMat negRot1M = cvMat(2, 3, CV_32FC1, _negRot1M); // Origin rotation matrix
        cv2DRotationMatrix( cvPoint2D32f(0,0), -angle, 1.0, &negRot1M);

        // Left and Down vectors
        float _right[3] = {float(r.width), 0, 1}, _right1[3];
        float _down[3] = {0, float(r.height), 1}, _down1[3];

        CvMat right = cvMat(3, 1, CV_32F, _right);
        CvMat right1 = cvMat(2, 1, CV_32F, _right1);
        CvMat down = cvMat(3, 1, CV_32F, _down);
        CvMat down1 = cvMat(2, 1, CV_32F, _down1);

        cvMatMul(&negRot1M, &right, &right1);
        cvMatMul(&negRot1M, &down, &down1);

        CvPoint TL = cvPoint(int(r.x - _right1[0]/2 - _down1[0]/2), int(r.y - _right1[1]/2 - _down1[1]/2));
        CvPoint TR = cvPoint(int(r.x + _right1[0]/2 - _down1[0]/2), int(r.y + _right1[1]/2 - _down1[1]/2));
        CvPoint BL = cvPoint(int(r.x - _right1[0]/2 + _down1[0]/2), int(r.y - _right1[1]/2 + _down1[1]/2));
        CvPoint BR = cvPoint(int(r.x + _right1[0]/2 + _down1[0]/2), int(r.y + _right1[1]/2 + _down1[1]/2));

        // Black outline
        if (outline)
        {
            cvLine(inputFrame, TL, TR, cvScalar(0,0,0), 5);
            cvLine(inputFrame, TR, BR, cvScalar(0,0,0), 5);
            cvLine(inputFrame, BR, BL, cvScalar(0,0,0), 5);
            cvLine(inputFrame, BL, TL, cvScalar(0,0,0), 5);
        }

        // line
        cvLine(inputFrame, TL, TR, color);
        cvLine(inputFrame, TR, BR, color);
        cvLine(inputFrame, BR, BL, color);
        cvLine(inputFrame, BL, TL, color);

        if (font && printResponse)
        {
            char responseStr[256];
            sprintf(responseStr, "%3.2f", response);
            CvPoint textPos = TL;
            textPos.y -= 5;
            cvPutText(inputFrame, responseStr, TL, font, color);
        }

    } // Result loop
}


// Print Graph@FIT tag
void tagImage(IplImage * image, const char * text, CvFont * font)
{
    CvSize sz;
    int baseline = 0;
    cvGetTextSize(text, font, &sz, &baseline);

    cvRectangle(image, cvPoint(0, image->height-(baseline+sz.height+5)), cvPoint(image->width, image->height), CV_RGB(20,20,20), CV_FILLED);

    cvPutText(image, text, cvPoint(8, image->height-(baseline)), font, CV_RGB(255,255,255));
}

Output * createOutput(const Config & c, const unsigned i, Input *in)
{
	Output * output = NULL;
	//open file output
	if(!c.output.empty())
	{
		ostringstream name(c.output);
		if(c.inputs.size() > 1) name << i;
		name << (c.inputs[i].type == Config::Input::IMAGE ? ".jpg" : ".avi");
		switch(c.inputs[i].type)
		{
		case Config::Input::IMAGE :
			name << ".jpg";
			output = new ImageOutput(name.str());
			break;
		default :
			name << ".avi";
			output = new VideoOutput(
				cvCreateVideoWriter(name.str().c_str(), c.fourcc,
					dynamic_cast<CaptureInput*>(in)->getProperty(CV_CAP_PROP_FPS)/c.step,
					cvGetSize(in->getFrame(0))));
		}
	}
	//open process output
	if(!c.process.empty())
	{
		vplDbProcessDelete(c.db, c.process.c_str());
		Output * processOutput = new ProcessOutput(c.db,
			vplDbProcessMng(c.db, c.process.c_str(), "CvBox2D", t_box2d),
			dynamic_cast<DatabaseInput*>(in));
		//if both outputs, compose them
		output = output == NULL ? processOutput : new CompositeOutput(output, processOutput);
	}
	//if no output, create null 
	return output == NULL ? new Output() : output;
}


int processFrame(IplImage *inputFrame, TDetectionList & groups, Detector & detector, const Config & config, const CvFont & font)
{
    // Flip image if needed
    if (inputFrame->origin)
    {
        inputFrame->origin = 0;
        cvFlip(inputFrame);
    }

    // FIXME: remove alloc - use preallocated images
    IplImage * inputGrayFrame = cvCreateImage(cvSize(inputFrame->width, inputFrame->height), IPL_DEPTH_8U, 1);

    //convert to grayscale
	cvCvtColor(inputFrame, inputGrayFrame, CV_RGB2GRAY);

    if (config.smooth > 0)
    {
        cvSmooth(inputGrayFrame, inputGrayFrame, CV_GAUSSIAN, 2*config.smooth+1);
    }

	TDetectionList detections;
	{
		TSCCounterGuard counter_guard(frameTSC);
		detector.detect(inputGrayFrame, detections);
	}

	if (config.drawAll)
    {
        drawDetections(inputFrame, detections.begin(), detections.end(), cvScalar(255,255,255, 128), false, false, 0);
    }

    // Non-maxima suppression
	groupDetections(detections, groups, config.groupMinCount, config.groupThreshold);

    drawDetections(inputFrame, groups.begin(), groups.end(), cvScalar(0,255,0), true, false, &font);

    cvReleaseImage(&inputGrayFrame);

    return groups.size();
}
*/

/*bool parseCommandLine(Config & config, int argc, char ** argv)
{
	// configuration in file instead of command line
	arg_file *config_file = arg_filen("C", "config", "<file>", 0, argc, "configuration file");
	// engine
	arg_str  *engine  = arg_str0("E", "engine", "<engine>", "detection engine");
	// classifier
	arg_file *classifier = arg_file0("c", "classifier", "<xml>", "classifier xml file");
	arg_dbl  *threshold  = arg_dbl0("t", "threshold", "<double>", "detection threshold");
	arg_int  *stages     = arg_int0(NULL, "stages", "<unsigned>", "maximal evaluated stage count");
	arg_lit  *noearly    = arg_lit0(NULL, "noearly", "disable early termination");
	arg_int  *hstep      = arg_int0(NULL, "hstep", "<unsigned>", "detection horizontal step");
	arg_int  *vstep      = arg_int0(NULL, "vstep", "<unsigned>", "detection vertical step");
	// nonmaxima supression
	arg_dbl  *groupThreshold = arg_dbl0("T", "group-threshold", "<double>", "detection overlap threshold");
	arg_int  *groupMinCount  = arg_int0("N", "goup-count", "<unsigned>", "group minimal detection count");
	// preprocessing
	arg_int  *smooth    = arg_int0(NULL, "smooth", "<unsigned>", "smoothing gaussian kernel size");
	arg_dbl  *scaleBase = arg_dbl0("b", "scale-base", "<double>", "starting image scale");
	// output
	arg_str  *output  = arg_str0("o", "output", "<output>", "output file name");
	arg_str  *detections = arg_str0(NULL, "detections", "<detections>", "detections file name");
	arg_str  *timestamps = arg_str0(NULL, "timestamps", "<timestaps>", "timestamps file name");
	arg_str  *fourcc  = arg_str0(NULL, "fourcc", "<FOURCC>", "FOURCC code of output file format");
	arg_lit  *tag     = arg_lit0(NULL, "tag", "tag output images");
	arg_lit  *show    = arg_lit0("s", "show", "show output in highgui window");
	arg_lit  *drawAll = arg_lit0("d", "draw-all", "draw all detections, not only groups");
	// database
	arg_str  *database    = arg_str0(NULL, "database", NULL, "database connection string");
	arg_str  *datapath    = arg_str0(NULL, "datapath", NULL, "data path prefix");
	arg_str  *dataset     = arg_str0(NULL, "dataset", NULL, "dataset name");
	arg_str  *process     = arg_str0(NULL, "process", NULL, "database output process name");
	arg_str  *description = arg_str0(NULL, "description", NULL, "database output process description");
	// misc
	arg_lit  *verbose = arg_lit0("v", "verbose", "verbose output");
	arg_lit  *help    = arg_lit0(NULL, "help", "print this help and exit");
	// input
	arg_int  *start  = arg_int0(NULL, "start", "<unsigned>", "starting frame");
	arg_int  *frames = arg_int0(NULL, "frames", "<unsigned>", "frame count");
	arg_int  *step   = arg_int0(NULL, "step", "<unsigned>", "frame step");
	arg_file *inputs = arg_filen(NULL, NULL, "{ <file> | CAM[n] }", 0, argc, "input files");
	// end
	struct arg_end *end = arg_end(20);
	//table
	void *argtable[] = {
		config_file,
		engine,
		classifier, threshold, stages, noearly, hstep, vstep,
		groupThreshold, groupMinCount,
		smooth, scaleBase,
		output, detections, timestamps, fourcc, tag, show, drawAll,
		database, datapath, dataset, process, description,
		verbose, help, 
		start, frames, step, inputs,
		end };
	bool retval = true;
	if(arg_nullcheck(argtable) != 0)
	{
		cerr << "Insufficient memory to parse arguments!\n";
		retval = false;
	}
	else
	{
		int errors = arg_parse(argc, argv, argtable);
		//print help
		if(help->count > 0)
		{
			cout << "Usage: abon";
			arg_print_syntax(stdout,argtable,"\n");
			arg_print_glossary_gnu(stdout,argtable);
			retval = false;
		}
		//check for errors
		else if(errors > 0)
		{
			arg_print_errors(stderr, end, "detect");
			cerr << "Try 'detect --help' for more information.\n";
			retval = false;
		}
	}
	if(retval == false)
	{
		arg_freetable(argtable, sizeof(argtable)/sizeof(argtable[0]));
		return false;
	}
	//parse config files
	for(int i = 0; i < config_file->count; ++i)
	{
		if(verbose->count) cout << "Parsing '" << config_file->filename[i] << "'...\n";
		if(!parseCommandFile(config, config_file->filename[i]))
		{
			retval = false;
		}
	}
	//macro for checking unsigned argument value
#define CONFIG_UNSIGNED(name) do {\
	if(name->count) {\
		if(name->ival[0] < 0) {\
			cerr << "Value of " << name->hdr.longopts << " mustn't be negative!\n"; \
			retval = false; }\
		config.name = name->ival[0]; } } while(false)
#define CONFIG_POSINT(name) do {\
	if(name->count) {\
		if(name->ival[0] <= 0) {\
			cerr << "Value of " << name->hdr.longopts << " must be greater than zero!\n"; \
			retval = false; }\
		config.name = name->ival[0]; } } while(false)
	//engine
	if(engine->count) config.engine  = engine->sval[0];
	//classifier
	if(classifier->count) config.classifier = classifier->filename[0];
	if(threshold->count) config.threshold = threshold->dval[0];
	CONFIG_UNSIGNED(stages);
	if(noearly->count) config.noearly = true;
	CONFIG_POSINT(hstep);
	CONFIG_POSINT(vstep);
	//group
	if(groupThreshold->count) config.groupThreshold = groupThreshold->dval[0];
	CONFIG_UNSIGNED(groupMinCount);
	//preprocess
	CONFIG_UNSIGNED(smooth);
	if(scaleBase->count) config.scaleBase = scaleBase->dval[0];
	//output
	if(output->count) config.output = output->sval[0];
	if(detections->count) config.detections = detections->sval[0];
	if(timestamps->count) config.timestamps = timestamps->sval[0];
	if(fourcc->count)
	{
		if(strlen(fourcc->sval[0]) != 4)
		{
			cerr << "FOURCC code must have four characters!\n";
			retval = false;
		}
		config.fourcc = CV_FOURCC(fourcc->sval[0][0], fourcc->sval[0][1], fourcc->sval[0][2], fourcc->sval[0][3]);
	}
	if(tag->count) config.tag = true;
	if(show->count) config.show = true;
	if(drawAll->count) config.drawAll = true;
	//database
	if(database->count) config.database = database->sval[0];
	if(datapath->count) config.datapath = datapath->sval[0];
	if(dataset->count) config.dataset = dataset->sval[0];
	if(process->count) config.process = process->sval[0];
	if(description->count) config.description = description->sval[0];
	//misc
	if(verbose->count) config.verbose = true;
	//input
	CONFIG_UNSIGNED(start);
	CONFIG_UNSIGNED(frames);
	CONFIG_UNSIGNED(step);
#undef CONFIG_UNSIGNED
#undef CONFIG_POSINT
	//input files
	for(int i = 0; i < inputs->count; ++i)
	{
		//determine input type
		const char *filename = inputs->filename[i];
		const char *extension = inputs->extension[i];
		if(strncmp(filename, "CAM", 3) == 0)
		{
			char * rest;
			long n = strtol(filename + 3, &rest, 10);
			if((*rest == '\0') && (n >= 0))
			{
				config.inputs.push_back(Config::Input(Config::Input::CAMERA, n));
			}
			else
			{
				cerr << "Invalid camera number in '" << filename << "'\n";
				retval = false;
			}
		}
/*		else if(strstr(filename, "DS") == filename)
		{
			config.inputs.push_back(Config::Input(Config::Input::DATASET));
		}
		else if(strstr(filename, "SEQ") == filename)
		{
			char * rest;
			long n = strtol(filename + 3, &rest, 10);
			if((*rest == '\0') && (n >= 0))
			{
				config.inputs.push_back(Config::Input(Config::Input::SEQUENCE, n));
			}
			else
			{
				cerr << "Invalid sequence number in '" << filename << "'\n";
				//return 0;
                goto error;
			}
		}* /
		else if((strcmp(extension, ".avi") == 0) || (strcmp(extension, ".mpg") == 0))
		{
			config.inputs.push_back(Config::Input(Config::Input::VIDEO, filename));
		}
		else
		{
			config.inputs.push_back(Config::Input(Config::Input::IMAGE, filename));
		}
	}
	//cleanup
	arg_freetable(argtable, sizeof(argtable)/sizeof(argtable[0]));
	return retval;
}

// Split text file to lines, convert to argv-like array and parse like command line
bool parseCommandFile(Config & config, const char * filename)
{
	//open config file
	ifstream file(filename, ios::in|ios::ate);
	if(!file)
	{
		cerr << "Can't open config file '" << filename << "'!\n";
		return false;
	}
	//read complete contents
	vector<char> contents(file.tellg());
	file.seekg(0, ios::beg);
	file.read(&contents[0], contents.size());
	//replace \n with \0 and collect pointers
	vector<char*> argv(0);
	unsigned first = 0;
	for(unsigned i = 0; i < contents.size(); ++i)
	{
		if(contents[i] == '\n')
		{
			contents[i] = '\0';
			//skip comments
			if(contents[first] != '#')
			{
				argv.push_back(&contents[first]);
			}
			first = i+1;
		}
	}
	argv.push_back(NULL);
	//parse as regular command line
	if(parseCommandLine(config, argv.size() - 1, &argv[0]))
	{
		return true;
	}
	else
	{
		cerr << "While parsing command file '" << filename << "'\n";
		return false;
	}
}
*/