#include "multidetector.h"

#include <algorithm>

using namespace std;

class abonOrderer : public abonDetector
{
public :
    abonDetector *detector;
    std::priority_queue<abonImageData *> output;
    unsigned next;

    abonOrderer(abonDetector *);
};

extern "C" { // interface -----------------------------------------

abonDetector *abonOrderOutput(abonDetector *detector)
{
    return new(nothrow) abonOrderer(detector);
}

abonErrorCode *ordererSetVerbosity(abonDetector *detector, unsigned verbosity)
{
    detector->verbosity = verbosity;
    return abonSetVerbosity(((abonOrderer*)detector)->detector, verbosity);
}

abonErrorCode *ordererSetTimer(abonDetector *detector, Timer *timer)
{
    detector->timer = timer;
    return abonSetTimer(((abonOrderer*)detector)->detector, timer);
}

abonErrorCode *ordererSetClassifier(abonDetector *detector, TClassifier *classifier)
{
    detector->classifier = classifier;
    return abonSetClassifier(((abonOrderer*)detector)->detector, classifier);
}

abonErrorCode *ordererSetStep(abonDetector *detector, unsigned h, unsigned v)
{
    detector->hstep = h;
    detector->vstep = v;
    return abonSetStep(((abonOrderer*)detector)->detector, h, v);
}

abonErrorCode *ordererSetSize(abonDetector *detector, unsigned w, unsigned h, unsigned p)
{
    detector->width = w;
    detector->height = h;
    detector->pitch = p;
    return abonSetSize(((abonOrderer*)detector)->detector, w, h, p);
}

abonErrorCode ordererProcess(abonDetector *detector, abonImageData *data)
{
    // process doesn't work...
    return aboneError;
}

abonErrorCode ordererStart(abonDetector *detector, abonCbGetImage *get, abonCbPutImage *put, void *param)
{
    // TODO : dodlat
}

abonErrorCode ordererRelease(abonDetector *detector)
{
    abonOrderer *orderer = (abonOrderer*)detector;
    abonErrorCode e = abonRelease(orderer->detector);
    delete orderer;
    return e;
}

} // interface -----------------------------------------

abonOrderer::abonOrderer(abonDetector *d)
: abonDetector(abonDetectorInitializer)
, detector(d), next(0)
{
    abonDetector::setVerbosity = ordererSetVerbosity;
    abonDetector::setTimer = ordererSetTimer;
    abonDetector::setClassifier = ordererSetClassifier;
    abonDetector::setStep = ordererSetStep;
    abonDetector::process = ordererProcess;
    abonDetector::start = ordererStart;
    abonDetector::release = ordererRelease;
}

#if 0
/*
abonErrorCode Multidetector::multiSetVerbosity(abonDetector *header, unsigned verbosity)
{
	Multidetector *detector = (Multidetector*)header;
	for(unsigned i = 0; i < detector->detectors.size(); ++i)
	{
		abonErrorCode ec = abonSetVerbosity(detector->detectors[i].detector, verbosity);
		if(ec != aboneOk)
			return ec;
	}
	return aboneOk;
}

abonErrorCode Multidetector::multiSetTimer(abonDetector *header, Timer *timer)
{
	Multidetector *detector = (Multidetector*)header;
	for(unsigned i = 0; i < detector->detectors.size(); ++i)
	{
		abonErrorCode ec = abonSetTimer(detector->detectors[i].detector, timer);
		if(ec != aboneOk)
			return ec;
	}
	return aboneOk;
}

abonErrorCode Multidetector::multiSetClassifier(abonDetector *header, TClassifier *classifier)
{
	Multidetector *detector = (Multidetector*)header;
	for(unsigned i = 0; i < detector->detectors.size(); ++i)
	{
		abonErrorCode ec = abonSetClassifier(detector->detectors[i].detector, classifier);
		if(ec != aboneOk)
			return ec;
	}
	return aboneOk;
}

abonErrorCode Multidetector::multiSetSize(abonDetector *header, unsigned width, unsigned height, unsigned pitch)
{
	Multidetector *detector = (Multidetector*)header;
	for(unsigned i = 0; i < detector->detectors.size(); ++i)
	{
		abonErrorCode ec = abonSetSize(detector->detectors[i].detector, width, height, pitch);
		if(ec != aboneOk)
			return ec;
	}
	return aboneOk;
}
*/

abonImageData *Multidetector::multiGetImage(void *param)
{
	Multidetector *detector = (Multidetector*)param;
/*	pthread_mutex_lock(&detector->inputMutex);
	while(detector->input.empty())
	{
		pthread_cond_wait(&detector->inputCond, &detector->inputMutex);
	}
	abonImageData *image = detector->input.front();
	if(image != NULL)
	{
		detector->input.pop();
	}
	pthread_mutex_unlock(&detector->inputMutex);
	return image;*/
	return NULL;
}

abonErrorCode Multidetector::multiPutImage(void *param, abonImageData *image)
{
	Multidetector *detector = (Multidetector*)param;
/*	pthread_mutex_lock(&detector->outputMutex);
	detector->output.push(image);
	pthread_cond_broadcast(&detector->outputCond);
	pthread_mutex_unlock(&detector->outputMutex);*/
	return aboneOk;
}

void *Multidetector::multiThreadProc(void *detector)
{
	abonStart(((Multidetector*)detector)->detector, multiGetImage, multiPutImage, detector);
	return NULL;
}

abonErrorCode Multidetector::multiStart(abonDetector *header, abonCbGetImage *get, abonCbPutImage *put, void *param)
{
	Multidetector *detector = (Multidetector*)header;
/*	//spawn threads
	for(unsigned i = 0; i < detector->detectors.size(); ++i)
	{
		detector->detectors[i].param = (void*)detector;
		pthread_create(&detector->detectors[i].thread, NULL, multiThreadProc, (void*)&detector->detectors[i]);
		
	}
	unsigned inputID = 0;
	unsigned outputID = 0;
*/
	pthread_create(&detector->thread, NULL, multiThreadProc, (void*)detector);
	return aboneOk;
}

Multidetector::Multidetector(abonDetector *d)
: abonDetector(abonDetectorInitializer)
, detector(d)
, inputMutex(PTHREAD_MUTEX_INITIALIZER)
, inputCond(PTHREAD_COND_INITIALIZER)
, outputMutex(PTHREAD_MUTEX_INITIALIZER)
, outputCond(PTHREAD_COND_INITIALIZER)
{
/*	setVerbosity  = multiSetVerbosity;
	setTimer      = multiSetTimer;
	setClassifier = multiSetClassifier;
	setSize       = multiSetSize;*/
	start = multiStart;
}

Multidetector::~Multidetector()
{
//	abonUnload(detector);
}
#endif
